/**
 * Define api of apeman-api-path
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @param {string} [options.root=process.cwd()] - Root directory path
 * @returns {Object} - API configuration
 */
'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var co = require('co');
var aslogger = require('aslogger');
var path = require('path');
var defaults = require('defaults');
var describe = require('./describe');

var cwd = function cwd() {
  return process.cwd();
};

/** @lends define */
function define() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var logger = aslogger({ disabled: !options.logging });

  var _defaults = defaults(options, {
    root: cwd()
  });

  var root = _defaults.root;


  return {
    ping: function ping() {
      var pong = arguments.length <= 0 || arguments[0] === undefined ? 'pong' : arguments[0];

      return co(regeneratorRuntime.mark(function _callee() {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                logger.trace('pong:', pong);
                return _context.abrupt('return', pong);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
    },
    dirname: function dirname(filename) {
      var s = this;
      var args = [].concat(Array.prototype.slice.call(arguments));
      return co(regeneratorRuntime.mark(function _callee2() {
        var dirname;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                dirname = path.dirname.apply(path, _toConsumableArray(args));
                _context2.next = 3;
                return s.resolve(dirname);

              case 3:
                return _context2.abrupt('return', _context2.sent);

              case 4:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
    },
    basename: function basename(filename, extname) {
      var s = this;
      var args = [].concat(Array.prototype.slice.call(arguments));
      return co(regeneratorRuntime.mark(function _callee3() {
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                return _context3.abrupt('return', path.basename.apply(path, _toConsumableArray(args)));

              case 1:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));
    },
    relative: function relative(from, to) {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee4() {
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return s.resolve(from);

              case 2:
                from = _context4.sent;
                _context4.next = 5;
                return s.resolve(to);

              case 5:
                to = _context4.sent;
                return _context4.abrupt('return', path.relative(from, to));

              case 7:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));
    },
    resolve: function resolve() {
      for (var _len = arguments.length, filenames = Array(_len), _key = 0; _key < _len; _key++) {
        filenames[_key] = arguments[_key];
      }

      var s = this;
      return co(regeneratorRuntime.mark(function _callee5() {
        var resolved, isTraversal;
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                resolved = path.resolve.apply(path, [root].concat(filenames));
                isTraversal = resolved.indexOf(root) !== 0;

                if (!isTraversal) {
                  _context5.next = 4;
                  break;
                }

                throw new Error('DIRECTORY_TRAVERSAL_DETECTED');

              case 4:
                return _context5.abrupt('return', path.relative(cwd(), resolved));

              case 5:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));
    },


    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before: function $before(methodName, params) {
      var s = this;
      var state = s.state;

      return co(regeneratorRuntime.mark(function _callee6() {
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                state.invokedAt = new Date();
                logger.info(methodName + ' started...');

              case 2:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));
    },

    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after: function $after(methodName, params, returns) {
      var s = this;
      var state = s.state;

      return co(regeneratorRuntime.mark(function _callee7() {
        var took;
        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                took = new Date() - state.invokedAt;

                logger.info('...' + methodName + ' done! (' + took + 'ms)');

              case 2:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));
    },

    /**
     * Method specifications
     */
    $spec: describe()
  };
}

module.exports = define;
//# sourceMappingURL=data:application/json;base64,bnVsbA==